const Backbone = require("backbone");
const _ = require("underscore");
const { ipcRenderer } = require("electron");
const {
    SessionImageModel,
    SessionSettingsModel,
    IntensityDatasetModel,
    TransmittanceDatasetModel,
    SessionComponentCollection,
    DatasetCollection
} = require("./models.js");
const spectroscopy = require("./spectroscopy.js");

let session = new SessionComponentCollection();
session.create({
    id: "smp",
    type: "image",
    title: "Sample Image"
});
session.create({
    id: "ref",
    type: "image",
    title: "Reference Image"
});
session.create({
    id: "settings",
    type: "settings"
});

let spectra = new DatasetCollection();
spectra.create({
    id: "smp",
    type: "intensity",
    title: "Sample Intensity"
});
spectra.create({
    id: "ref",
    type: "intensity",
    title: "Reference Intensity"
});
spectra.create({
    id: "transmit",
    type: "transmittance",
    title: "Transmittance"
});

let smpImage = session.get("smp");
let refImage = session.get("ref");
let settings = session.get("settings");

let smpIntensity = spectra.get("smp");
let refIntensity = spectra.get("ref");
let transmittance = spectra.get("transmit");
var data = [];
var data_smp=[];
var data_ref=[];
var data_ta=[];
var flag=0;
var flag_smp=0;
var flag_ref=0;
let transmitOnChange = function (model, options) {
    this.set(spectroscopy.calculateTransmittance(smpIntensity, refIntensity));
    console.log(flag);

    var lengths=spectroscopy.calculateTransmittance(smpIntensity, refIntensity).nm.length;

    for (var i = 0; i < lengths; i++)
      data_ta[i]= ","+ spectroscopy.calculateTransmittance(smpIntensity, refIntensity).nm[i]+","+spectroscopy.calculateTransmittance(smpIntensity, refIntensity).t[i]+","+spectroscopy.calculateTransmittance(smpIntensity, refIntensity).a[i];

    this.save();
}

let imgOnChange = function (model, options) {
    let spectrum = spectra.get(this.get("id"));
    var id =this.get("id");
    let blueNM = model.get("blueNM");
    let redNM = model.get("redNM");
    console.log(redNM-blueNM);
    if (this.isReady()) {
        let specWidth = session.get("settings").get("specWidth");
        spectroscopy.calculateIntensity(this, specWidth, function (intensity) {
            spectrum.set(intensity);
            data[0]="Smp Wavelength"+","+"Smp R"+","+"Smp G"+","+"Smp B"+","+"Ref Wavelength"+","+"Ref R"+","+"Ref G"+","+"Ref B"+","+"Wavelength"+","+"Transmittance"+","+"Absorbance";
            if(id=="smp")
              flag_smp=1;
            else if (id=="ref")
              flag_ref=1;

            if (id=="smp"){
              for (var i = 0; i < intensity.nm.length; i++)
                data_smp[i]="\n"+intensity.nm[i]+","+intensity.r[i]+","+intensity.g[i]+","+intensity.b[i];
            }

            if (id=="ref"){
              for (var i = 0; i < intensity.nm.length; i++)
                data_ref[i]=","+intensity.nm[i]+","+intensity.r[i]+","+intensity.g[i]+","+intensity.b[i];
            }

            if (flag_smp!=0 && flag_ref!=0)
            {
              for (var i = 0; i < intensity.nm.length; i++)
                data[i+1]=data_smp[i]+data_ref[i]+data_ta[i];
              ipcRenderer.send("save-sessiondata",data,0);
            }

            ipcRenderer.send("printD",flag_smp,flag_ref);
            spectrum.save();
        });
    } else {
        spectrum.set({
            nm: [],
            r: [],
            g: [],
            b: [],
            sum: []
        });
        spectrum.save();

        //spectrum.get will get the properties of the model
    }
    // ipcRenderer.send("download-file",data,0);
}

transmittance.listenTo(smpIntensity, "change", transmitOnChange);
transmittance.listenTo(refIntensity, "change", transmitOnChange);

smpImage.on("change", imgOnChange);
smpImage.listenTo(settings, "change:specWidth", imgOnChange);
refImage.on("change", imgOnChange);
refImage.listenTo(settings, "change:specWidth", imgOnChange);




ipcRenderer.on("session-change", function (event) {
    console.log("Session Changed!");
    session.fetch();
});

ipcRenderer.send("compute-ready");
